@@ -389,7 +389,7 @@ static const char *blkid_probe_get_probername(blkid_probe pr)
 {
 	struct blkid_chain *chn = blkid_probe_get_chain(pr);
 
-	if (chn && chn->idx >= 0 && chn->idx < chn->driver->nidinfos)
+	if (chn && chn->idx >= 0 && (unsigned)chn->idx < chn->driver->nidinfos)
 		return chn->driver->idinfos[chn->idx]->name;
 
 	return NULL;
@@ -580,17 +580,78 @@ int __blkid_probe_filter_types(blkid_probe pr, int chain, int flag, char *names[
 	return 0;
 }
 
-unsigned char *blkid_probe_get_buffer(blkid_probe pr,
-				blkid_loff_t off, blkid_loff_t len)
+static struct blkid_bufinfo *read_buffer(blkid_probe pr, uint64_t real_off, uint64_t len)
+{
+	ssize_t ret;
+	struct blkid_bufinfo *bf = NULL;
+
+	if (blkid_llseek(pr->fd, real_off, SEEK_SET) < 0) {
+		errno = 0;
+		return NULL;
+	}
+
+	/* someone trying to overflow some buffers? */
+	if (len > ULONG_MAX - sizeof(struct blkid_bufinfo)) {
+		errno = ENOMEM;
+		return NULL;
+	}
+
+	/* allocate info and space for data by one malloc call */
+	bf = calloc(1, sizeof(struct blkid_bufinfo) + len);
+	if (!bf) {
+		errno = ENOMEM;
+		return NULL;
+	}
+
+	bf->data = ((unsigned char *) bf) + sizeof(struct blkid_bufinfo);
+	bf->len = len;
+	bf->off = real_off;
+	INIT_LIST_HEAD(&bf->bufs);
+
+	DBG(LOWPROBE, ul_debug("\tread %p: off=%"PRIu64" len=%"PRIu64"",
+	                       bf->data, real_off, len));
+
+	ret = read(pr->fd, bf->data, len);
+	if (ret != (ssize_t) len) {
+		DBG(LOWPROBE, ul_debug("\tread failed: %m"));
+		free(bf);
+
+		/* I/O errors on CDROMs are non-fatal to work with hybrid
+		 * audio+data disks */
+		if (ret >= 0 || blkid_probe_is_cdrom(pr))
+			errno = 0;
+		return NULL;
+	}
+
+	return bf;
+}
+
+/*
+ * Note that @off is offset within probing area, the probing area is defined by
+ * pr->off and pr->size.
+ */
+unsigned char *blkid_probe_get_buffer(blkid_probe pr, uint64_t off, uint64_t len)
 {
 	struct list_head *p;
 	struct blkid_bufinfo *bf = NULL;
+	uint64_t real_off = pr->off + off;
 
-	if (pr->size <= 0) {
+	/*
+	DBG(BUFFER, ul_debug("\t>>>> off=%ju, real-off=%ju (probe <%ju..%ju>, len=%ju",
+				off, real_off, pr->off, pr->off + pr->size, len));
+	*/
+
+	if (pr->size == 0) {
 		errno = EINVAL;
 		return NULL;
 	}
 
+	if (len == 0 || pr->off + pr->size < real_off + len) {
+		DBG(BUFFER, ul_debug("\t  ignore: request out of probing area"));
+		errno = 0;
+		return NULL;
+	}
+
 	if (pr->parent &&
 	    pr->parent->devno == pr->devno &&
 	    pr->parent->off <= pr->off &&
@@ -600,88 +661,64 @@ unsigned char *blkid_probe_get_buffer(blkid_probe pr,
 		 * parent. Let's use parent's buffers.
 		 *
 		 * Note that pr->off (and pr->parent->off) is always from the
-		 * beginig of the device.
+		 * begin of the device.
 		 */
 		return blkid_probe_get_buffer(pr->parent,
 				pr->off + off - pr->parent->off, len);
 	}
 
+	/* try buffers we already have in memmory */
 	list_for_each(p, &pr->buffers) {
 		struct blkid_bufinfo *x =
 				list_entry(p, struct blkid_bufinfo, bufs);
 
-		if (x->off <= off && off + len <= x->off + x->len) {
-			DBG(LOWPROBE, ul_debug("\treuse buffer: off=%jd len=%jd pr=%p",
-							x->off, x->len, pr));
+		if (real_off >= x->off && real_off + len <= x->off + x->len) {
+			DBG(BUFFER, ul_debug("\treuse %p: off=%"PRIu64" len=%"PRIu64" (for off=%"PRIu64" len=%"PRIu64")",
+						x->data, x->off, x->len, real_off, len));
 			bf = x;
 			break;
 		}
 	}
+
+	/* not found; read from disk */
 	if (!bf) {
-		ssize_t ret;
-
-		if (blkid_llseek(pr->fd, pr->off + off, SEEK_SET) < 0) {
-			errno = 0;
+		bf = read_buffer(pr, real_off, len);
+		if (!bf)
 			return NULL;
-		}
 
-		/* someone trying to overflow some buffers? */
-		if (len > ULONG_MAX - sizeof(struct blkid_bufinfo)) {
-			errno = ENOMEM;
-			return NULL;
-		}
-
-		/* allocate info and space for data by why call */
-		bf = calloc(1, sizeof(struct blkid_bufinfo) + len);
-		if (!bf) {
-			errno = ENOMEM;
-			return NULL;
-		}
-
-		bf->data = ((unsigned char *) bf) + sizeof(struct blkid_bufinfo);
-		bf->len = len;
-		bf->off = off;
-		INIT_LIST_HEAD(&bf->bufs);
-
-		DBG(LOWPROBE, ul_debug("\tbuffer read: off=%jd len=%jd pr=%p",
-				off, len, pr));
-
-		ret = read(pr->fd, bf->data, len);
-		if (ret != (ssize_t) len) {
-			DBG(LOWPROBE, ul_debug("\tbuffer read: return %zd error %m", ret));
-			free(bf);
-			if (ret >= 0)
-				errno = 0;
-			return NULL;
-		}
 		list_add_tail(&bf->bufs, &pr->buffers);
 	}
 
+	assert(bf->off <= real_off);
+	assert(bf->off + bf->len >= real_off + len);
+
 	errno = 0;
-	return off ? bf->data + (off - bf->off) : bf->data;
+	return real_off ? bf->data + (real_off - bf->off) : bf->data;
 }
 
 static void blkid_probe_reset_buffer(blkid_probe pr)
 {
-	uint64_t read_ct = 0, len_ct = 0;
+	uint64_t ct = 0, len = 0;
 
 	if (!pr || list_empty(&pr->buffers))
 		return;
 
-	DBG(LOWPROBE, ul_debug("reseting probing buffers pr=%p", pr));
+	DBG(BUFFER, ul_debug("reseting probing buffers pr=%p", pr));
 
 	while (!list_empty(&pr->buffers)) {
 		struct blkid_bufinfo *bf = list_entry(pr->buffers.next,
 						struct blkid_bufinfo, bufs);
-		read_ct++;
-		len_ct += bf->len;
+		ct++;
+		len += bf->len;
 		list_del(&bf->bufs);
+
+		DBG(BUFFER, ul_debug(" remove buffer: %p [off=%"PRIu64", len=%"PRIu64"]",
+		                     bf->data, bf->off, bf->len));
 		free(bf);
 	}
 
-	DBG(LOWPROBE, ul_debug("buffers summary: %"PRIu64" bytes "
-			"by %"PRIu64" read() call(s)",
-			len_ct, read_ct));
+	DBG(LOWPROBE, ul_debug(" buffers summary: %"PRIu64" bytes by %"PRIu64" read() calls",
+			len, ct));
 
 	INIT_LIST_HEAD(&pr->buffers);
 }
@@ -734,6 +771,7 @@ int blkid_probe_set_device(blkid_probe pr, int fd,
 		blkid_loff_t off, blkid_loff_t size)
 {
 	struct stat sb;
+	uint64_t devsiz = 0;
 
 	if (!pr)
 		return -1;
@@ -749,7 +787,7 @@ int blkid_probe_set_device(blkid_probe pr, int fd,
 	pr->flags &= ~BLKID_FL_CDROM_DEV;
 	pr->prob_flags = 0;
 	pr->fd = fd;
-	pr->off = off;
+	pr->off = (uint64_t) off;
 	pr->size = 0;
 	pr->devno = 0;
 	pr->disk_devno = 0;
@@ -766,33 +804,35 @@ int blkid_probe_set_device(blkid_probe pr, int fd,
 	if (fstat(fd, &sb))
 		goto err;
 
-	if (!S_ISBLK(sb.st_mode) && !S_ISCHR(sb.st_mode) && !S_ISREG(sb.st_mode))
+	if (!S_ISBLK(sb.st_mode) && !S_ISCHR(sb.st_mode) && !S_ISREG(sb.st_mode)) {
+		errno = EINVAL;
 		goto err;
-
+	}
 
 	pr->mode = sb.st_mode;
 	if (S_ISBLK(sb.st_mode) || S_ISCHR(sb.st_mode))
 		pr->devno = sb.st_rdev;
 
-	if (size)
-		pr->size = size;
-	else {
-		if (S_ISBLK(sb.st_mode)) {
-			if (blkdev_get_size(fd, (unsigned long long *) &pr->size)) {
-				DBG(LOWPROBE, ul_debug("failed to get device size"));
-				goto err;
-			}
-		} else if (S_ISCHR(sb.st_mode))
-			pr->size = 1;		/* UBI devices are char... */
-		else if (S_ISREG(sb.st_mode))
-			pr->size = sb.st_size;	/* regular file */
-
-		if (pr->off > pr->size)
+	if (S_ISBLK(sb.st_mode)) {
+		if (blkdev_get_size(fd, (unsigned long long *) &devsiz)) {
+			DBG(LOWPROBE, ul_debug("failed to get device size"));
 			goto err;
+		}
+	} else if (S_ISCHR(sb.st_mode))
+		devsiz = 1;		/* UBI devices are char... */
+	else if (S_ISREG(sb.st_mode))
+		devsiz = sb.st_size;	/* regular file */
 
-		/* The probing area cannot be larger than whole device, pr->off
-		 * is offset within the device */
-		pr->size -= pr->off;
+	pr->size = size ? (uint64_t)size : devsiz;
+
+	if (off && size == 0)
+		/* only offset without size specified */
+		pr->size -= (uint64_t) off;
+
+	if (pr->off + pr->size > devsiz) {
+		DBG(LOWPROBE, ul_debug("area specified by offset and size is bigger than device"));
+		errno = EINVAL;
+		goto err;
 	}
 
 	if (pr->size <= 1440 * 1024 && !S_ISCHR(sb.st_mode))
@@ -811,7 +851,7 @@ int blkid_probe_set_device(blkid_probe pr, int fd,
 		pr->flags |= BLKID_FL_CDROM_DEV;
 #endif
 
-	DBG(LOWPROBE, ul_debug("ready for low-probing, offset=%jd, size=%jd",
+	DBG(LOWPROBE, ul_debug("ready for low-probing, offset=%"PRIu64", size=%"PRIu64"",
 				pr->off, pr->size));
 	DBG(LOWPROBE, ul_debug("whole-disk: %s, regfile: %s",
 		blkid_probe_is_wholedisk(pr) ?"YES" : "NO",
@@ -824,8 +864,7 @@ err:
 
 }
 
-int blkid_probe_get_dimension(blkid_probe pr,
-		blkid_loff_t *off, blkid_loff_t *size)
+int blkid_probe_get_dimension(blkid_probe pr, uint64_t *off, uint64_t *size)
 {
 	if (!pr)
 		return -1;
@@ -835,26 +874,21 @@ int blkid_probe_get_dimension(blkid_probe pr,
 	return 0;
 }
 
-int blkid_probe_set_dimension(blkid_probe pr,
-		blkid_loff_t off, blkid_loff_t size)
+int blkid_probe_set_dimension(blkid_probe pr, uint64_t off, uint64_t size)
 {
 	if (!pr)
 		return -1;
 
 	DBG(LOWPROBE, ul_debug(
-		"changing probing area pr=%p: size=%llu, off=%llu "
-		"-to-> size=%llu, off=%llu",
-		pr,
-		(unsigned long long) pr->size,
-		(unsigned long long) pr->off,
-		(unsigned long long) size,
-		(unsigned long long) off));
+		"changing probing area pr=%p: size=%"PRIu64", off=%"PRIu64" "
+		"-to-> size=%"PRIu64", off=%"PRIu64"",
+		pr, pr->size, pr->off, size, off));
 
 	pr->off = off;
 	pr->size = size;
 	pr->flags &= ~BLKID_FL_TINY_DEV;
 
-	if (pr->size <= 1440 * 1024 && !S_ISCHR(pr->mode))
+	if (pr->size <= 1440ULL * 1024ULL && !S_ISCHR(pr->mode))
 		pr->flags |= BLKID_FL_TINY_DEV;
 
 	blkid_probe_reset_buffer(pr);
@@ -868,10 +902,10 @@ int blkid_probe_set_dimension(blkid_probe pr,
  * or no magic present, or negative value on error.
  */
 int blkid_probe_get_idmag(blkid_probe pr, const struct blkid_idinfo *id,
-			blkid_loff_t *offset, const struct blkid_idmag **res)
+			uint64_t *offset, const struct blkid_idmag **res)
 {
 	const struct blkid_idmag *mag = NULL;
-	blkid_loff_t off = 0;
+	uint64_t off = 0;
 
 	if (id)
 		mag = &id->magics[0];
@@ -887,8 +921,10 @@ int blkid_probe_get_idmag(blkid_probe pr, const struct blkid_idinfo *id,
 
 		if (!buf && errno)
 			return -errno;
+
 		if (buf && !memcmp(mag->magic,
 				buf + (mag->sboff & 0x3ff), mag->len)) {
+
 			DBG(LOWPROBE, ul_debug("\tmagic sboff=%u, kboff=%ld",
 				mag->sboff, mag->kboff));
 			if (offset)
@@ -1058,7 +1094,7 @@ int blkid_do_wipe(blkid_probe pr, int dryrun)
 {
 	const char *off = NULL;
 	size_t len = 0;
-	loff_t offset, l;
+	uint64_t offset, l;
 	char buf[BUFSIZ];
 	int fd, rc = 0;
 	struct blkid_chain *chn;
@@ -1088,7 +1124,7 @@ int blkid_do_wipe(blkid_probe pr, int dryrun)
 	if (rc || len == 0 || off == NULL)
 		return 0;
 
-	offset = strtoll(off, NULL, 10);
+	offset = strtoumax(off, NULL, 10) + pr->off;
 	fd = blkid_probe_get_fd(pr);
 	if (fd < 0)
 		return -1;
@@ -1097,11 +1133,11 @@ int blkid_do_wipe(blkid_probe pr, int dryrun)
 		len = sizeof(buf);
 
 	DBG(LOWPROBE, ul_debug(
-	    "do_wipe [offset=0x%jx, len=%zd, chain=%s, idx=%d, dryrun=%s]\n",
-	    offset, len, chn->driver->name, chn->idx, dryrun ? "yes" : "not"));
+	    "do_wipe [offset=0x%"PRIx64" (%"PRIu64"), len=%zu, chain=%s, idx=%d, dryrun=%s]\n",
+	    offset, offset, len, chn->driver->name, chn->idx, dryrun ? "yes" : "not"));
 
-	l = lseek(fd, offset, SEEK_SET);
-	if (l == (off_t) -1)
+	l = blkid_llseek(fd, offset, SEEK_SET);
+	if ((blkid_loff_t)l == (off_t) -1)
 		return -1;
 
 	memset(buf, 0, len);
@@ -1325,7 +1361,7 @@ done:
 unsigned char *blkid_probe_get_sector(blkid_probe pr, unsigned int sector)
 {
 	return pr ? blkid_probe_get_buffer(pr,
-			((blkid_loff_t) sector) << 9, 0x200) : NULL;
+			((uint64_t) sector) << 9, 0x200) : NULL;
 }
 
 struct blkid_prval *blkid_probe_assign_value(
@@ -1359,8 +1395,8 @@ static struct blkid_prval *blkid_probe_new_value(void)
 }
 
 /* Note that value data is always terminated by zero to keep things robust,
- * this extra zero is not count to the value lenght. It's caller responsibility
- * to set proper value lenght (for strings we count terminator to the lenght,
+ * this extra zero is not count to the value length. It's caller responsibility
+ * to set proper value length (for strings we count terminator to the length,
  * for binary data it's without terminator).
  */
 int blkid_probe_value_set_data(struct blkid_prval *v,
@@ -1420,7 +1456,7 @@ int blkid_probe_sprintf_value(blkid_probe pr, const char *name,
 	return rc;
 }
 
-int blkid_probe_set_magic(blkid_probe pr, blkid_loff_t offset,
+int blkid_probe_set_magic(blkid_probe pr, uint64_t offset,
 			size_t len, unsigned char *magic)
 {
 	int rc = 0;
@@ -1460,7 +1496,7 @@ int blkid_probe_verify_csum(blkid_probe pr, uint64_t csum, uint64_t expected)
 
 		DBG(LOWPROBE, ul_debug(
 				"incorrect checksum for type %s,"
-				" got %jX, expected %jX",
+				" got %"PRIX64", expected %"PRIX64"",
 				blkid_probe_get_probername(pr),
 				csum, expected));
 		/*
@@ -1582,7 +1618,7 @@ blkid_probe blkid_probe_get_wholedisk_probe(blkid_probe pr)
  */
 blkid_loff_t blkid_probe_get_size(blkid_probe pr)
 {
-	return pr ? pr->size : -1;
+	return pr ? (blkid_loff_t) pr->size : -1;
 }
 
 /**
@@ -1595,7 +1631,7 @@ blkid_loff_t blkid_probe_get_size(blkid_probe pr)
  */
 blkid_loff_t blkid_probe_get_offset(blkid_probe pr)
 {
-	return pr ? pr->off : -1;
+	return pr ? (blkid_loff_t) pr->off : -1;
 }
 
 /**
@@ -1639,7 +1675,7 @@ unsigned int blkid_probe_get_sectorsize(blkid_probe pr)
  */
 blkid_loff_t blkid_probe_get_sectors(blkid_probe pr)
 {
-	return pr ? pr->size >> 9 : -1;
+	return pr ? (blkid_loff_t) (pr->size >> 9) : -1;
 }
 
 /**
@@ -1854,7 +1890,7 @@ size_t blkid_ltrim_whitespace(unsigned char *str)
  * Note that there is not relation between _wiper and blkid_to_wipe().
  *
  */
-void blkid_probe_set_wiper(blkid_probe pr, blkid_loff_t off, blkid_loff_t size)
+void blkid_probe_set_wiper(blkid_probe pr, uint64_t off, uint64_t size)
 {
 	struct blkid_chain *chn;
 
@@ -1879,7 +1915,7 @@ void blkid_probe_set_wiper(blkid_probe pr, blkid_loff_t off, blkid_loff_t size)
 	pr->wipe_chain = chn;
 
 	DBG(LOWPROBE,
-		ul_debug("wiper set to %s::%s off=%jd size=%jd",
+		ul_debug("wiper set to %s::%s off=%"PRIu64" size=%"PRIu64"",
 			chn->driver->name,
 			chn->driver->idinfos[chn->idx]->name,
 			pr->wipe_off, pr->wipe_size));
@@ -1889,8 +1925,7 @@ void blkid_probe_set_wiper(blkid_probe pr, blkid_loff_t off, blkid_loff_t size)
 /*
  * Returns 1 if the <@off,@size> area was wiped
  */
-int blkid_probe_is_wiped(blkid_probe pr, struct blkid_chain **chn,
-		     blkid_loff_t off, blkid_loff_t size)
+int blkid_probe_is_wiped(blkid_probe pr, struct blkid_chain **chn, uint64_t off, uint64_t size)
 {
 	if (!pr || !size)
 		return 0;
@@ -1907,7 +1942,7 @@ int blkid_probe_is_wiped(blkid_probe pr, struct blkid_chain **chn,
  *  Try to use any area -- if the area has been previously wiped then the
  *  previous probing result should be ignored (reseted).
  */
-void blkid_probe_use_wiper(blkid_probe pr, blkid_loff_t off, blkid_loff_t size)
+void blkid_probe_use_wiper(blkid_probe pr, uint64_t off, uint64_t size)
 {
 	struct blkid_chain *chn = NULL;
 
